import logging
import os
from logging.handlers import RotatingFileHandler

import tomli
from akmi_utils.logging import log_requests
from akmi_utils.otel import PrometheusMiddleware, metrics, setting_otlp, MetricsEndpointFilter, TraceContextFilter
from fastapi import FastAPI, Request


def get_project_details(base_dir: str, keys: list):
    with open(os.path.join(base_dir, 'pyproject.toml'), 'rb') as file:
        package_details = tomli.load(file)
    poetry = package_details['project']
    return {key: poetry[key] for key in keys}

def set_otlp(app: FastAPI, app_name: str, otlp_grpc_endpoint: str, log_file: str, log_config: dict):
    app.add_middleware(PrometheusMiddleware, app_name=app_name)
    app.add_route("/metrics", metrics)
    setting_otlp(app, app_name, otlp_grpc_endpoint)
    logging.getLogger("uvicorn.access").addFilter(MetricsEndpointFilter())
    logging.getLogger("uvicorn.access").addFilter(TraceContextFilter())
    log_config["formatters"]["access"]["fmt"] = (
        "%(asctime)s %(levelname)s [%(name)s] [%(filename)s:%(lineno)d] [%(funcName)s] "
        "[trace_id=%(otelTraceID)s span_id=%(otelSpanID)s resource.service.name=%(otelServiceName)s] - %(message)s"
    )
    file_handler = RotatingFileHandler(log_file, maxBytes=10 * 1024 * 1024, backupCount=10)
    file_handler.setFormatter(logging.Formatter(log_config["formatters"]["access"]["fmt"]))
    logging.getLogger().addHandler(file_handler)
    # Set the logging level for h11 to ERROR
    logging.getLogger("h11").setLevel(logging.ERROR)
    file_handler.setLevel(logging.INFO)

    @app.middleware("http")
    async def log_requests_middleware(request: Request, call_next):
        return await log_requests(request, call_next)